from typing import ClassVar as _ClassVar
from typing import Iterable as _Iterable
from typing import Mapping as _Mapping
from typing import Optional as _Optional
from typing import Union as _Union

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper

import apibara.starknet.proto.types_pb2 as _types_pb2

BLOCK_STATUS_ACCEPTED_ON_L1: BlockStatus
BLOCK_STATUS_ACCEPTED_ON_L2: BlockStatus
BLOCK_STATUS_PENDING: BlockStatus
BLOCK_STATUS_REJECTED: BlockStatus
BLOCK_STATUS_UNSPECIFIED: BlockStatus
DESCRIPTOR: _descriptor.FileDescriptor

class Block(_message.Message):
    __slots__ = [
        "events",
        "header",
        "l2_to_l1_messages",
        "state_update",
        "status",
        "transactions",
    ]
    EVENTS_FIELD_NUMBER: _ClassVar[int]
    HEADER_FIELD_NUMBER: _ClassVar[int]
    L2_TO_L1_MESSAGES_FIELD_NUMBER: _ClassVar[int]
    STATE_UPDATE_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    TRANSACTIONS_FIELD_NUMBER: _ClassVar[int]
    events: _containers.RepeatedCompositeFieldContainer[EventWithTransaction]
    header: BlockHeader
    l2_to_l1_messages: _containers.RepeatedCompositeFieldContainer[
        L2ToL1MessageWithTransaction
    ]
    state_update: StateUpdate
    status: BlockStatus
    transactions: _containers.RepeatedCompositeFieldContainer[TransactionWithReceipt]
    def __init__(
        self,
        status: _Optional[_Union[BlockStatus, str]] = ...,
        header: _Optional[_Union[BlockHeader, _Mapping]] = ...,
        transactions: _Optional[
            _Iterable[_Union[TransactionWithReceipt, _Mapping]]
        ] = ...,
        state_update: _Optional[_Union[StateUpdate, _Mapping]] = ...,
        events: _Optional[_Iterable[_Union[EventWithTransaction, _Mapping]]] = ...,
        l2_to_l1_messages: _Optional[
            _Iterable[_Union[L2ToL1MessageWithTransaction, _Mapping]]
        ] = ...,
    ) -> None: ...

class BlockHeader(_message.Message):
    __slots__ = [
        "block_hash",
        "block_number",
        "new_root",
        "parent_block_hash",
        "sequencer_address",
        "timestamp",
    ]
    BLOCK_HASH_FIELD_NUMBER: _ClassVar[int]
    BLOCK_NUMBER_FIELD_NUMBER: _ClassVar[int]
    NEW_ROOT_FIELD_NUMBER: _ClassVar[int]
    PARENT_BLOCK_HASH_FIELD_NUMBER: _ClassVar[int]
    SEQUENCER_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    block_hash: _types_pb2.FieldElement
    block_number: int
    new_root: _types_pb2.FieldElement
    parent_block_hash: _types_pb2.FieldElement
    sequencer_address: _types_pb2.FieldElement
    timestamp: _timestamp_pb2.Timestamp
    def __init__(
        self,
        block_hash: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        parent_block_hash: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        block_number: _Optional[int] = ...,
        sequencer_address: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        new_root: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        timestamp: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
    ) -> None: ...

class DeclareTransaction(_message.Message):
    __slots__ = ["class_hash", "sender_address"]
    CLASS_HASH_FIELD_NUMBER: _ClassVar[int]
    SENDER_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    class_hash: _types_pb2.FieldElement
    sender_address: _types_pb2.FieldElement
    def __init__(
        self,
        class_hash: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        sender_address: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
    ) -> None: ...

class DeclaredContract(_message.Message):
    __slots__ = ["class_hash"]
    CLASS_HASH_FIELD_NUMBER: _ClassVar[int]
    class_hash: _types_pb2.FieldElement
    def __init__(
        self, class_hash: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...
    ) -> None: ...

class DeployAccountTransaction(_message.Message):
    __slots__ = ["class_hash", "constructor_calldata", "contract_address_salt"]
    CLASS_HASH_FIELD_NUMBER: _ClassVar[int]
    CONSTRUCTOR_CALLDATA_FIELD_NUMBER: _ClassVar[int]
    CONTRACT_ADDRESS_SALT_FIELD_NUMBER: _ClassVar[int]
    class_hash: _types_pb2.FieldElement
    constructor_calldata: _containers.RepeatedCompositeFieldContainer[
        _types_pb2.FieldElement
    ]
    contract_address_salt: _types_pb2.FieldElement
    def __init__(
        self,
        constructor_calldata: _Optional[
            _Iterable[_Union[_types_pb2.FieldElement, _Mapping]]
        ] = ...,
        contract_address_salt: _Optional[
            _Union[_types_pb2.FieldElement, _Mapping]
        ] = ...,
        class_hash: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
    ) -> None: ...

class DeployTransaction(_message.Message):
    __slots__ = ["class_hash", "constructor_calldata", "contract_address_salt"]
    CLASS_HASH_FIELD_NUMBER: _ClassVar[int]
    CONSTRUCTOR_CALLDATA_FIELD_NUMBER: _ClassVar[int]
    CONTRACT_ADDRESS_SALT_FIELD_NUMBER: _ClassVar[int]
    class_hash: _types_pb2.FieldElement
    constructor_calldata: _containers.RepeatedCompositeFieldContainer[
        _types_pb2.FieldElement
    ]
    contract_address_salt: _types_pb2.FieldElement
    def __init__(
        self,
        constructor_calldata: _Optional[
            _Iterable[_Union[_types_pb2.FieldElement, _Mapping]]
        ] = ...,
        contract_address_salt: _Optional[
            _Union[_types_pb2.FieldElement, _Mapping]
        ] = ...,
        class_hash: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
    ) -> None: ...

class DeployedContract(_message.Message):
    __slots__ = ["class_hash", "contract_address"]
    CLASS_HASH_FIELD_NUMBER: _ClassVar[int]
    CONTRACT_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    class_hash: _types_pb2.FieldElement
    contract_address: _types_pb2.FieldElement
    def __init__(
        self,
        contract_address: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        class_hash: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
    ) -> None: ...

class Event(_message.Message):
    __slots__ = ["data", "from_address", "keys"]
    DATA_FIELD_NUMBER: _ClassVar[int]
    FROM_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    KEYS_FIELD_NUMBER: _ClassVar[int]
    data: _containers.RepeatedCompositeFieldContainer[_types_pb2.FieldElement]
    from_address: _types_pb2.FieldElement
    keys: _containers.RepeatedCompositeFieldContainer[_types_pb2.FieldElement]
    def __init__(
        self,
        from_address: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        keys: _Optional[_Iterable[_Union[_types_pb2.FieldElement, _Mapping]]] = ...,
        data: _Optional[_Iterable[_Union[_types_pb2.FieldElement, _Mapping]]] = ...,
    ) -> None: ...

class EventWithTransaction(_message.Message):
    __slots__ = ["event", "receipt", "transaction"]
    EVENT_FIELD_NUMBER: _ClassVar[int]
    RECEIPT_FIELD_NUMBER: _ClassVar[int]
    TRANSACTION_FIELD_NUMBER: _ClassVar[int]
    event: Event
    receipt: TransactionReceipt
    transaction: Transaction
    def __init__(
        self,
        transaction: _Optional[_Union[Transaction, _Mapping]] = ...,
        receipt: _Optional[_Union[TransactionReceipt, _Mapping]] = ...,
        event: _Optional[_Union[Event, _Mapping]] = ...,
    ) -> None: ...

class InvokeTransactionV0(_message.Message):
    __slots__ = ["calldata", "contract_address", "entry_point_selector"]
    CALLDATA_FIELD_NUMBER: _ClassVar[int]
    CONTRACT_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    ENTRY_POINT_SELECTOR_FIELD_NUMBER: _ClassVar[int]
    calldata: _containers.RepeatedCompositeFieldContainer[_types_pb2.FieldElement]
    contract_address: _types_pb2.FieldElement
    entry_point_selector: _types_pb2.FieldElement
    def __init__(
        self,
        contract_address: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        entry_point_selector: _Optional[
            _Union[_types_pb2.FieldElement, _Mapping]
        ] = ...,
        calldata: _Optional[_Iterable[_Union[_types_pb2.FieldElement, _Mapping]]] = ...,
    ) -> None: ...

class InvokeTransactionV1(_message.Message):
    __slots__ = ["calldata", "sender_address"]
    CALLDATA_FIELD_NUMBER: _ClassVar[int]
    SENDER_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    calldata: _containers.RepeatedCompositeFieldContainer[_types_pb2.FieldElement]
    sender_address: _types_pb2.FieldElement
    def __init__(
        self,
        sender_address: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        calldata: _Optional[_Iterable[_Union[_types_pb2.FieldElement, _Mapping]]] = ...,
    ) -> None: ...

class L1HandlerTransaction(_message.Message):
    __slots__ = ["calldata", "contract_address", "entry_point_selector"]
    CALLDATA_FIELD_NUMBER: _ClassVar[int]
    CONTRACT_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    ENTRY_POINT_SELECTOR_FIELD_NUMBER: _ClassVar[int]
    calldata: _containers.RepeatedCompositeFieldContainer[_types_pb2.FieldElement]
    contract_address: _types_pb2.FieldElement
    entry_point_selector: _types_pb2.FieldElement
    def __init__(
        self,
        contract_address: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        entry_point_selector: _Optional[
            _Union[_types_pb2.FieldElement, _Mapping]
        ] = ...,
        calldata: _Optional[_Iterable[_Union[_types_pb2.FieldElement, _Mapping]]] = ...,
    ) -> None: ...

class L2ToL1Message(_message.Message):
    __slots__ = ["payload", "to_address"]
    PAYLOAD_FIELD_NUMBER: _ClassVar[int]
    TO_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    payload: _containers.RepeatedCompositeFieldContainer[_types_pb2.FieldElement]
    to_address: _types_pb2.FieldElement
    def __init__(
        self,
        to_address: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        payload: _Optional[_Iterable[_Union[_types_pb2.FieldElement, _Mapping]]] = ...,
    ) -> None: ...

class L2ToL1MessageWithTransaction(_message.Message):
    __slots__ = ["message", "receipt", "transaction"]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    RECEIPT_FIELD_NUMBER: _ClassVar[int]
    TRANSACTION_FIELD_NUMBER: _ClassVar[int]
    message: L2ToL1Message
    receipt: TransactionReceipt
    transaction: Transaction
    def __init__(
        self,
        transaction: _Optional[_Union[Transaction, _Mapping]] = ...,
        receipt: _Optional[_Union[TransactionReceipt, _Mapping]] = ...,
        message: _Optional[_Union[L2ToL1Message, _Mapping]] = ...,
    ) -> None: ...

class NonceUpdate(_message.Message):
    __slots__ = ["contract_address", "nonce"]
    CONTRACT_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    NONCE_FIELD_NUMBER: _ClassVar[int]
    contract_address: _types_pb2.FieldElement
    nonce: _types_pb2.FieldElement
    def __init__(
        self,
        contract_address: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        nonce: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
    ) -> None: ...

class StateDiff(_message.Message):
    __slots__ = ["declared_contracts", "deployed_contracts", "nonces", "storage_diffs"]
    DECLARED_CONTRACTS_FIELD_NUMBER: _ClassVar[int]
    DEPLOYED_CONTRACTS_FIELD_NUMBER: _ClassVar[int]
    NONCES_FIELD_NUMBER: _ClassVar[int]
    STORAGE_DIFFS_FIELD_NUMBER: _ClassVar[int]
    declared_contracts: _containers.RepeatedCompositeFieldContainer[DeclaredContract]
    deployed_contracts: _containers.RepeatedCompositeFieldContainer[DeployedContract]
    nonces: _containers.RepeatedCompositeFieldContainer[NonceUpdate]
    storage_diffs: _containers.RepeatedCompositeFieldContainer[StorageDiff]
    def __init__(
        self,
        storage_diffs: _Optional[_Iterable[_Union[StorageDiff, _Mapping]]] = ...,
        declared_contracts: _Optional[
            _Iterable[_Union[DeclaredContract, _Mapping]]
        ] = ...,
        deployed_contracts: _Optional[
            _Iterable[_Union[DeployedContract, _Mapping]]
        ] = ...,
        nonces: _Optional[_Iterable[_Union[NonceUpdate, _Mapping]]] = ...,
    ) -> None: ...

class StateUpdate(_message.Message):
    __slots__ = ["new_root", "old_root", "state_diff"]
    NEW_ROOT_FIELD_NUMBER: _ClassVar[int]
    OLD_ROOT_FIELD_NUMBER: _ClassVar[int]
    STATE_DIFF_FIELD_NUMBER: _ClassVar[int]
    new_root: _types_pb2.FieldElement
    old_root: _types_pb2.FieldElement
    state_diff: StateDiff
    def __init__(
        self,
        new_root: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        old_root: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        state_diff: _Optional[_Union[StateDiff, _Mapping]] = ...,
    ) -> None: ...

class StorageDiff(_message.Message):
    __slots__ = ["contract_address", "storage_entries"]
    CONTRACT_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    STORAGE_ENTRIES_FIELD_NUMBER: _ClassVar[int]
    contract_address: _types_pb2.FieldElement
    storage_entries: _containers.RepeatedCompositeFieldContainer[StorageEntry]
    def __init__(
        self,
        contract_address: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        storage_entries: _Optional[_Iterable[_Union[StorageEntry, _Mapping]]] = ...,
    ) -> None: ...

class StorageEntry(_message.Message):
    __slots__ = ["key", "value"]
    KEY_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    key: _types_pb2.FieldElement
    value: _types_pb2.FieldElement
    def __init__(
        self,
        key: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        value: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
    ) -> None: ...

class Transaction(_message.Message):
    __slots__ = [
        "declare",
        "deploy",
        "deploy_account",
        "invoke_v0",
        "invoke_v1",
        "l1_handler",
        "meta",
    ]
    DECLARE_FIELD_NUMBER: _ClassVar[int]
    DEPLOY_ACCOUNT_FIELD_NUMBER: _ClassVar[int]
    DEPLOY_FIELD_NUMBER: _ClassVar[int]
    INVOKE_V0_FIELD_NUMBER: _ClassVar[int]
    INVOKE_V1_FIELD_NUMBER: _ClassVar[int]
    L1_HANDLER_FIELD_NUMBER: _ClassVar[int]
    META_FIELD_NUMBER: _ClassVar[int]
    declare: DeclareTransaction
    deploy: DeployTransaction
    deploy_account: DeployAccountTransaction
    invoke_v0: InvokeTransactionV0
    invoke_v1: InvokeTransactionV1
    l1_handler: L1HandlerTransaction
    meta: TransactionMeta
    def __init__(
        self,
        meta: _Optional[_Union[TransactionMeta, _Mapping]] = ...,
        invoke_v0: _Optional[_Union[InvokeTransactionV0, _Mapping]] = ...,
        invoke_v1: _Optional[_Union[InvokeTransactionV1, _Mapping]] = ...,
        deploy: _Optional[_Union[DeployTransaction, _Mapping]] = ...,
        declare: _Optional[_Union[DeclareTransaction, _Mapping]] = ...,
        l1_handler: _Optional[_Union[L1HandlerTransaction, _Mapping]] = ...,
        deploy_account: _Optional[_Union[DeployAccountTransaction, _Mapping]] = ...,
    ) -> None: ...

class TransactionMeta(_message.Message):
    __slots__ = ["hash", "max_fee", "nonce", "signature", "version"]
    HASH_FIELD_NUMBER: _ClassVar[int]
    MAX_FEE_FIELD_NUMBER: _ClassVar[int]
    NONCE_FIELD_NUMBER: _ClassVar[int]
    SIGNATURE_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    hash: _types_pb2.FieldElement
    max_fee: _types_pb2.FieldElement
    nonce: _types_pb2.FieldElement
    signature: _containers.RepeatedCompositeFieldContainer[_types_pb2.FieldElement]
    version: int
    def __init__(
        self,
        hash: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        max_fee: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        signature: _Optional[
            _Iterable[_Union[_types_pb2.FieldElement, _Mapping]]
        ] = ...,
        nonce: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        version: _Optional[int] = ...,
    ) -> None: ...

class TransactionReceipt(_message.Message):
    __slots__ = [
        "actual_fee",
        "events",
        "l2_to_l1_messages",
        "transaction_hash",
        "transaction_index",
    ]
    ACTUAL_FEE_FIELD_NUMBER: _ClassVar[int]
    EVENTS_FIELD_NUMBER: _ClassVar[int]
    L2_TO_L1_MESSAGES_FIELD_NUMBER: _ClassVar[int]
    TRANSACTION_HASH_FIELD_NUMBER: _ClassVar[int]
    TRANSACTION_INDEX_FIELD_NUMBER: _ClassVar[int]
    actual_fee: _types_pb2.FieldElement
    events: _containers.RepeatedCompositeFieldContainer[Event]
    l2_to_l1_messages: _containers.RepeatedCompositeFieldContainer[L2ToL1Message]
    transaction_hash: _types_pb2.FieldElement
    transaction_index: int
    def __init__(
        self,
        transaction_hash: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        transaction_index: _Optional[int] = ...,
        actual_fee: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        l2_to_l1_messages: _Optional[_Iterable[_Union[L2ToL1Message, _Mapping]]] = ...,
        events: _Optional[_Iterable[_Union[Event, _Mapping]]] = ...,
    ) -> None: ...

class TransactionWithReceipt(_message.Message):
    __slots__ = ["receipt", "transaction"]
    RECEIPT_FIELD_NUMBER: _ClassVar[int]
    TRANSACTION_FIELD_NUMBER: _ClassVar[int]
    receipt: TransactionReceipt
    transaction: Transaction
    def __init__(
        self,
        transaction: _Optional[_Union[Transaction, _Mapping]] = ...,
        receipt: _Optional[_Union[TransactionReceipt, _Mapping]] = ...,
    ) -> None: ...

class BlockStatus(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from dataclasses import dataclass
from typing import Generic, Optional, TypeVar

from apibara.protocol.proto.stream_pb2 import Cursor, DataFinality

Data = TypeVar("Data")
Filter = TypeVar("Filter")


@dataclass
class IndexerConfiguration(Generic[Filter]):
    filter: Filter
    starting_cursor: Optional[Cursor] = None
    finality: Optional[DataFinality.ValueType] = None



from dataclasses import dataclass
from typing import Generic, Optional, TypeVar

from apibara.indexer.storage import Filter, Storage
from apibara.protocol.proto.stream_pb2 import Cursor

UserContext = TypeVar("UserContext")


@dataclass
class Info(Generic[UserContext, Filter]):
    """State shared between handlers.

    Parameters
    ----------
    context:
        application-specific context.
    storage:
        access the chain-aware storage.
    """

    context: UserContext
    storage: Storage
    cursor: Cursor
    end_cursor: Cursor

    _new_filter: Optional[Filter] = None

    def merge_filter(self, filter: Filter):
        """Add the new filter by merging it with the old filter.

        The indexer will re-scan the current block for any data
        matching the new filter.
        """
        self._new_filter = filter

    def _take_new_filter(self):
        """Returns the new filter, if set by the caller."""
        filter = self._new_filter
        self._new_filter = None
        return filter



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: starknet.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b"\n\x0estarknet.proto\x12\x19\x61pibara.starknet.v1alpha2\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x0btypes.proto\"\x93\x03\n\x05\x42lock\x12\x36\n\x06status\x18\x01 \x01(\x0e\x32&.apibara.starknet.v1alpha2.BlockStatus\x12\x36\n\x06header\x18\x02 \x01(\x0b\x32&.apibara.starknet.v1alpha2.BlockHeader\x12G\n\x0ctransactions\x18\x03 \x03(\x0b\x32\x31.apibara.starknet.v1alpha2.TransactionWithReceipt\x12<\n\x0cstate_update\x18\x04 \x01(\x0b\x32&.apibara.starknet.v1alpha2.StateUpdate\x12?\n\x06\x65vents\x18\x05 \x03(\x0b\x32/.apibara.starknet.v1alpha2.EventWithTransaction\x12R\n\x11l2_to_l1_messages\x18\x06 \x03(\x0b\x32\x37.apibara.starknet.v1alpha2.L2ToL1MessageWithTransaction\"\xd2\x02\n\x0b\x42lockHeader\x12;\n\nblock_hash\x18\x01 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x42\n\x11parent_block_hash\x18\x02 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x14\n\x0c\x62lock_number\x18\x03 \x01(\x04\x12\x42\n\x11sequencer_address\x18\x04 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x39\n\x08new_root\x18\x05 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12-\n\ttimestamp\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\x95\x01\n\x16TransactionWithReceipt\x12;\n\x0btransaction\x18\x01 \x01(\x0b\x32&.apibara.starknet.v1alpha2.Transaction\x12>\n\x07receipt\x18\x02 \x01(\x0b\x32-.apibara.starknet.v1alpha2.TransactionReceipt\"\xf8\x03\n\x0bTransaction\x12\x38\n\x04meta\x18\x01 \x01(\x0b\x32*.apibara.starknet.v1alpha2.TransactionMeta\x12\x43\n\tinvoke_v0\x18\x02 \x01(\x0b\x32..apibara.starknet.v1alpha2.InvokeTransactionV0H\x00\x12\x43\n\tinvoke_v1\x18\x03 \x01(\x0b\x32..apibara.starknet.v1alpha2.InvokeTransactionV1H\x00\x12>\n\x06\x64\x65ploy\x18\x04 \x01(\x0b\x32,.apibara.starknet.v1alpha2.DeployTransactionH\x00\x12@\n\x07\x64\x65\x63lare\x18\x05 \x01(\x0b\x32-.apibara.starknet.v1alpha2.DeclareTransactionH\x00\x12\x45\n\nl1_handler\x18\x06 \x01(\x0b\x32/.apibara.starknet.v1alpha2.L1HandlerTransactionH\x00\x12M\n\x0e\x64\x65ploy_account\x18\x07 \x01(\x0b\x32\x33.apibara.starknet.v1alpha2.DeployAccountTransactionH\x00\x42\r\n\x0btransaction\"\x87\x02\n\x0fTransactionMeta\x12\x35\n\x04hash\x18\x01 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x38\n\x07max_fee\x18\x02 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12:\n\tsignature\x18\x03 \x03(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x36\n\x05nonce\x18\x04 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x0f\n\x07version\x18\x05 \x01(\x04\"\xda\x01\n\x13InvokeTransactionV0\x12\x41\n\x10\x63ontract_address\x18\x01 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x45\n\x14\x65ntry_point_selector\x18\x02 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x39\n\x08\x63\x61lldata\x18\x03 \x03(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\"\x91\x01\n\x13InvokeTransactionV1\x12?\n\x0esender_address\x18\x01 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x39\n\x08\x63\x61lldata\x18\x02 \x03(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\"\xdf\x01\n\x11\x44\x65ployTransaction\x12\x45\n\x14\x63onstructor_calldata\x18\x02 \x03(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x46\n\x15\x63ontract_address_salt\x18\x03 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12;\n\nclass_hash\x18\x04 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\"\x92\x01\n\x12\x44\x65\x63lareTransaction\x12;\n\nclass_hash\x18\x01 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12?\n\x0esender_address\x18\x02 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\"\xdb\x01\n\x14L1HandlerTransaction\x12\x41\n\x10\x63ontract_address\x18\x02 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x45\n\x14\x65ntry_point_selector\x18\x03 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x39\n\x08\x63\x61lldata\x18\x04 \x03(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\"\xe6\x01\n\x18\x44\x65ployAccountTransaction\x12\x45\n\x14\x63onstructor_calldata\x18\x02 \x03(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x46\n\x15\x63ontract_address_salt\x18\x03 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12;\n\nclass_hash\x18\x04 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\"\xa6\x02\n\x12TransactionReceipt\x12\x41\n\x10transaction_hash\x18\x01 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x19\n\x11transaction_index\x18\x02 \x01(\x04\x12;\n\nactual_fee\x18\x03 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x43\n\x11l2_to_l1_messages\x18\x04 \x03(\x0b\x32(.apibara.starknet.v1alpha2.L2ToL1Message\x12\x30\n\x06\x65vents\x18\x05 \x03(\x0b\x32 .apibara.starknet.v1alpha2.Event\"\xd6\x01\n\x1cL2ToL1MessageWithTransaction\x12;\n\x0btransaction\x18\x01 \x01(\x0b\x32&.apibara.starknet.v1alpha2.Transaction\x12>\n\x07receipt\x18\x02 \x01(\x0b\x32-.apibara.starknet.v1alpha2.TransactionReceipt\x12\x39\n\x07message\x18\x03 \x01(\x0b\x32(.apibara.starknet.v1alpha2.L2ToL1Message\"\x86\x01\n\rL2ToL1Message\x12;\n\nto_address\x18\x03 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x38\n\x07payload\x18\x04 \x03(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\"\xc4\x01\n\x14\x45ventWithTransaction\x12;\n\x0btransaction\x18\x01 \x01(\x0b\x32&.apibara.starknet.v1alpha2.Transaction\x12>\n\x07receipt\x18\x02 \x01(\x0b\x32-.apibara.starknet.v1alpha2.TransactionReceipt\x12/\n\x05\x65vent\x18\x03 \x01(\x0b\x32 .apibara.starknet.v1alpha2.Event\"\xb4\x01\n\x05\x45vent\x12=\n\x0c\x66rom_address\x18\x01 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x35\n\x04keys\x18\x02 \x03(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x35\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\"\xbd\x01\n\x0bStateUpdate\x12\x39\n\x08new_root\x18\x01 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x39\n\x08old_root\x18\x02 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x38\n\nstate_diff\x18\x03 \x01(\x0b\x32$.apibara.starknet.v1alpha2.StateDiff\"\x94\x02\n\tStateDiff\x12=\n\rstorage_diffs\x18\x01 \x03(\x0b\x32&.apibara.starknet.v1alpha2.StorageDiff\x12G\n\x12\x64\x65\x63lared_contracts\x18\x02 \x03(\x0b\x32+.apibara.starknet.v1alpha2.DeclaredContract\x12G\n\x12\x64\x65ployed_contracts\x18\x03 \x03(\x0b\x32+.apibara.starknet.v1alpha2.DeployedContract\x12\x36\n\x06nonces\x18\x04 \x03(\x0b\x32&.apibara.starknet.v1alpha2.NonceUpdate\"\x92\x01\n\x0bStorageDiff\x12\x41\n\x10\x63ontract_address\x18\x01 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12@\n\x0fstorage_entries\x18\x02 \x03(\x0b\x32'.apibara.starknet.v1alpha2.StorageEntry\"|\n\x0cStorageEntry\x12\x34\n\x03key\x18\x01 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x36\n\x05value\x18\x02 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\"O\n\x10\x44\x65\x63laredContract\x12;\n\nclass_hash\x18\x01 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\"\x92\x01\n\x10\x44\x65ployedContract\x12\x41\n\x10\x63ontract_address\x18\x01 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12;\n\nclass_hash\x18\x02 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\"\x88\x01\n\x0bNonceUpdate\x12\x41\n\x10\x63ontract_address\x18\x01 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement\x12\x36\n\x05nonce\x18\x02 \x01(\x0b\x32'.apibara.starknet.v1alpha2.FieldElement*\xa2\x01\n\x0b\x42lockStatus\x12\x1c\n\x18\x42LOCK_STATUS_UNSPECIFIED\x10\x00\x12\x18\n\x14\x42LOCK_STATUS_PENDING\x10\x01\x12\x1f\n\x1b\x42LOCK_STATUS_ACCEPTED_ON_L2\x10\x02\x12\x1f\n\x1b\x42LOCK_STATUS_ACCEPTED_ON_L1\x10\x03\x12\x19\n\x15\x42LOCK_STATUS_REJECTED\x10\x04\x62\x06proto3"
)

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "starknet_pb2", globals())
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _BLOCKSTATUS._serialized_start = 5111
    _BLOCKSTATUS._serialized_end = 5273
    _BLOCK._serialized_start = 92
    _BLOCK._serialized_end = 495
    _BLOCKHEADER._serialized_start = 498
    _BLOCKHEADER._serialized_end = 836
    _TRANSACTIONWITHRECEIPT._serialized_start = 839
    _TRANSACTIONWITHRECEIPT._serialized_end = 988
    _TRANSACTION._serialized_start = 991
    _TRANSACTION._serialized_end = 1495
    _TRANSACTIONMETA._serialized_start = 1498
    _TRANSACTIONMETA._serialized_end = 1761
    _INVOKETRANSACTIONV0._serialized_start = 1764
    _INVOKETRANSACTIONV0._serialized_end = 1982
    _INVOKETRANSACTIONV1._serialized_start = 1985
    _INVOKETRANSACTIONV1._serialized_end = 2130
    _DEPLOYTRANSACTION._serialized_start = 2133
    _DEPLOYTRANSACTION._serialized_end = 2356
    _DECLARETRANSACTION._serialized_start = 2359
    _DECLARETRANSACTION._serialized_end = 2505
    _L1HANDLERTRANSACTION._serialized_start = 2508
    _L1HANDLERTRANSACTION._serialized_end = 2727
    _DEPLOYACCOUNTTRANSACTION._serialized_start = 2730
    _DEPLOYACCOUNTTRANSACTION._serialized_end = 2960
    _TRANSACTIONRECEIPT._serialized_start = 2963
    _TRANSACTIONRECEIPT._serialized_end = 3257
    _L2TOL1MESSAGEWITHTRANSACTION._serialized_start = 3260
    _L2TOL1MESSAGEWITHTRANSACTION._serialized_end = 3474
    _L2TOL1MESSAGE._serialized_start = 3477
    _L2TOL1MESSAGE._serialized_end = 3611
    _EVENTWITHTRANSACTION._serialized_start = 3614
    _EVENTWITHTRANSACTION._serialized_end = 3810
    _EVENT._serialized_start = 3813
    _EVENT._serialized_end = 3993
    _STATEUPDATE._serialized_start = 3996
    _STATEUPDATE._serialized_end = 4185
    _STATEDIFF._serialized_start = 4188
    _STATEDIFF._serialized_end = 4464
    _STORAGEDIFF._serialized_start = 4467
    _STORAGEDIFF._serialized_end = 4613
    _STORAGEENTRY._serialized_start = 4615
    _STORAGEENTRY._serialized_end = 4739
    _DECLAREDCONTRACT._serialized_start = 4741
    _DECLAREDCONTRACT._serialized_end = 4820
    _DEPLOYEDCONTRACT._serialized_start = 4823
    _DEPLOYEDCONTRACT._serialized_end = 4969
    _NONCEUPDATE._serialized_start = 4972
    _NONCEUPDATE._serialized_end = 5108
# @@protoc_insertion_point(module_scope)



import asyncio
import logging
from dataclasses import dataclass
from typing import Any, Generic, List, Optional, Tuple

from grpc import ssl_channel_credentials
from grpc.aio import insecure_channel, secure_channel

from apibara.indexer.indexer import Indexer
from apibara.indexer.info import Info, UserContext
from apibara.indexer.storage import Filter, IndexerStorage
from apibara.protocol import StreamService, credentials_with_auth_token
from apibara.protocol.proto.stream_pb2 import DataFinality, Cursor

logger = logging.getLogger(__name__)


MAX_MESSAGE_LENGTH = 100 * 1024 * 1024

DEFAULT_CLIENT_OPTIONS = [
    ("grpc.max_send_message_length", MAX_MESSAGE_LENGTH),
    ("grpc.max_receive_message_length", MAX_MESSAGE_LENGTH),
]


@dataclass
class IndexerRunnerConfiguration:
    """IndexerRunner configuration.

    Parameters
    ----------
    stream_url:
        url of the Apibara stream.
    stream_ssl:
        flag to connect using SSL.
    token:
        server authorization token.
    storage_url:
        MongoDB connection string, used to store the indexer  data and state.
    """

    stream_url: Optional[str] = None
    stream_ssl: bool = True
    token: Optional[str] = None
    storage_url: Optional[str] = None


class IndexerRunner(Generic[UserContext, Filter]):
    """Run an indexer, listening for new data and calling the provided callbacks.

    Parameters
    ----------
    reset_state:
        flag to restart the indexer from the beginning.
    config:
        options to set the input stream and connection string.
    client_options:
        list of options passed to the gRPC channel.
    """

    def __init__(
        self,
        *,
        reset_state: bool = False,
        config: Optional[IndexerRunnerConfiguration] = None,
        client_options: Optional[List[Tuple[str, Any]]] = None,
    ) -> None:
        if config is None:
            config = IndexerRunnerConfiguration()
        if client_options is None:
            client_options = DEFAULT_CLIENT_OPTIONS
        self._reset_state = reset_state
        self._config = config
        self._indexer_id = None
        self._indexer_storage = None
        self._client_options = client_options

    async def run(self, indexer: Indexer, *, ctx: Optional[UserContext] = None):
        """Run the indexer until stopped."""
        self._check_config()
        self._setup_storage(indexer)
        self._maybe_reset_state()
        await self._do_run(indexer, ctx)

    def _check_config(self):
        if self._config is None:
            raise RuntimeError("must provide config to IndexerRunner")

        if self._config.stream_url is None:
            raise RuntimeError("must provide a stream_url in config")

        if self._config.storage_url is None:
            raise RuntimeError("must provide a storage_url in config")

    def _setup_storage(self, indexer: Indexer):
        self._indexer_id = indexer.indexer_id()
        self._indexer_storage = IndexerStorage(
            self._config.storage_url, self._indexer_id
        )

    def _maybe_reset_state(self):
        if self._reset_state:
            logger.debug("reset state")
            self._indexer_storage.drop_database()

    async def _do_run(self, indexer: Indexer, ctx: Optional[UserContext] = None):
        self._retry_count = 0

        logger.debug("starting indexer")
        while True:
            try:
                await self._connect_and_stream(indexer, ctx)
            except Exception as exc:
                logger.debug(f"indexer exception {exc}")
                self._retry_count += 1
                reconnect = await indexer.handle_reconnect(exc, self._retry_count)

                if not reconnect.reconnect:
                    raise

    async def _connect_and_stream(self, indexer: Indexer, ctx: Optional[UserContext]):
        channel = self._channel()
        (client, stream) = StreamService(channel).stream_data()

        config = indexer.initial_configuration()
        has_stored = self._indexer_storage.update_with_stored_configuration(config)
        if has_stored:
            # invalidate old pending data, if any
            self._indexer_storage.invalidate(config.starting_cursor)

        logger.debug("indexer configuration read")

        await client.configure(
            filter=config.filter.encode(),
            finality=config.finality,
            cursor=config.starting_cursor,
            batch_size=1,
        )

        logger.debug("indexer configuration sent")

        previous_end_cursor = None
        pending_received = False
        additional_filter = None
        runner_state = "default"  # or "resync"

        async for message in stream:
            logger.debug("received message")
            self._retry_count = 0

            if message.data is not None:
                assert (
                    len(message.data.data) <= 1
                ), "indexer runner requires batch_size == 1"

                if runner_state == "resync":
                    logger.debug("handle block resync")
                    end_cursor = message.data.end_cursor
                    cursor = message.data.cursor
                    logger.debug(f"handle resync batch {cursor} - {end_cursor}")
                    with self._indexer_storage.create_storage_for_data(
                        message.data.end_cursor
                    ) as storage:
                        for batch in message.data.data:
                            decoded_data = indexer.decode_data(batch)
                            # only call handler if the batch is for the same block
                            if (
                                decoded_data.header.block_number
                                == previous_end_cursor.order_key
                            ):
                                info = Info(
                                    context=ctx,
                                    storage=storage,
                                    cursor=cursor,
                                    end_cursor=end_cursor,
                                )

                                await indexer.handle_data(info, decoded_data)
                                new_additional_filter = indexer._get_and_reset_filter()
                                if new_additional_filter is not None:
                                    raise RuntimeError(
                                        "additional filter not supported when rescanning block"
                                    )
                    # in any case, restart syncing from where it left off
                    runner_state = "default"
                    config.filter = config.filter.merge(additional_filter)
                    self._indexer_storage._update_filter(config.filter)
                    await client.configure(
                        filter=config.filter.encode(),
                        finality=config.finality,
                        cursor=previous_end_cursor,
                        batch_size=1,
                    )
                    continue

                # invalidate any pending data, if any
                if pending_received and previous_end_cursor is not None:
                    self._indexer_storage.invalidate(previous_end_cursor)

                is_pending = message.data.finality == DataFinality.DATA_STATUS_PENDING
                pending_received = is_pending

                end_cursor = message.data.end_cursor
                cursor = message.data.cursor

                logger.debug(f"handle batch {cursor} - {end_cursor}")

                with self._indexer_storage.create_storage_for_data(
                    message.data.end_cursor
                ) as storage:
                    additional_filter = None
                    for batch in message.data.data:
                        decoded_data = indexer.decode_data(batch)
                        info = Info(
                            context=ctx,
                            storage=storage,
                            cursor=cursor,
                            end_cursor=end_cursor,
                        )
                        if is_pending:
                            await indexer.handle_pending_data(info, decoded_data)
                            additional_filter = indexer._get_and_reset_filter()
                            if additional_filter is not None:
                                raise RuntimeError(
                                    "additional filter not supported for pending data"
                                )
                        else:
                            await indexer.handle_data(info, decoded_data)
                            additional_filter = indexer._get_and_reset_filter()

                        if additional_filter is not None:
                            logger.debug(
                                f"filter updated. rescanning block from {end_cursor.order_key - 1}"
                            )
                            runner_state = "resync"
                            await client.configure(
                                filter=additional_filter.encode(),
                                finality=config.finality,
                                cursor=Cursor(order_key=end_cursor.order_key - 1),
                                batch_size=1,
                            )

                if not is_pending:
                    previous_end_cursor = message.data.end_cursor

            elif message.invalidate is not None:
                with self._indexer_storage.create_storage_for_invalidate(
                    message.invalidate.cursor
                ) as storage:
                    cursor = message.invalidate.cursor
                    info = Info(
                        context=ctx, storage=storage, cursor=cursor, end_cursor=cursor
                    )

                    self._indexer_storage.invalidate(cursor, session=storage._session)

                    await indexer.handle_invalidate(info, cursor)
                previous_end_cursor = message.invalidate.cursor

    def _channel(self):
        if self._config.stream_ssl:
            return secure_channel(
                self._config.stream_url,
                credentials_with_auth_token(
                    self._config.token, ssl_channel_credentials()
                ),
                options=self._client_options,
            )
        return insecure_channel(self._config.stream_url, options=self._client_options)



import logging
from contextlib import contextmanager
from typing import Any, Generic, Iterable, Iterator, Optional, TypeVar

from pymongo import MongoClient
from pymongo.client_session import ClientSession
from pymongo.database import Database

import apibara.cursor as cursor_utils
from apibara.indexer.configuration import IndexerConfiguration
from apibara.protocol.proto.stream_pb2 import Cursor

Document = dict[str, Any]
DocumentFilter = dict[str, Any]
Update = dict[str, Any]
Projection = dict[str, any]

Filter = TypeVar("Filter")


logger = logging.getLogger(__name__)


class IndexerStorage(Generic[Filter]):
    """
    Manage indexers storage.
    """

    def __init__(self, url: Optional[str], indexer_id: str) -> None:
        if url is None:
            raise ValueError("Storage url must be not None")

        self.db_name = indexer_id.replace("-", "_")
        self._indexer_id = indexer_id

        self._mongo = MongoClient(url)
        self.db = self._mongo[self.db_name]

    @contextmanager
    def create_storage_for_block(self, cursor: Cursor) -> Iterator["Storage"]:
        with self._mongo.start_session() as session:
            yield Storage(self.db, session=session, cursor=cursor)

    @contextmanager
    def create_storage_for_data(self, cursor: Cursor) -> Iterator["Storage"]:
        with self._mongo.start_session() as session:
            yield Storage(self.db, session=session, cursor=cursor)
            self._update_cursor(cursor, session)

    @contextmanager
    def create_storage_for_invalidate(self, cursor: Cursor) -> Iterator["Storage"]:
        with self._mongo.start_session() as session:
            yield Storage(self.db, session=session, cursor=cursor)
            self._update_cursor(cursor, session)

    @contextmanager
    def create_storage_for_pending(self, cursor: Cursor) -> Iterator["Storage"]:
        with self._mongo.start_session() as session:
            yield Storage(self.db, session=session, cursor=cursor)

    def _initialize_configuration(self, configuration: IndexerConfiguration[Filter]):
        existing = self.db["_apibara"].find_one({"indexer_id": self._indexer_id})
        if existing is not None:
            return

        logger.debug("writing initial configuration to storage")

        cursor = None
        if configuration.starting_cursor is not None:
            cursor = cursor_utils.to_json(configuration.starting_cursor)

        if configuration.filter is None:
            raise RuntimeError("configuration filter must be defined")

        filter = configuration.filter.encode()

        self.db["_apibara"].insert_one(
            {
                "indexer_id": self._indexer_id,
                "cursor": cursor,
                "filter": filter,
            }
        )

    def update_with_stored_configuration(self, initial: IndexerConfiguration[Filter]):
        logger.debug("update config with stored version")

        stored = self.db["_apibara"].find_one({"indexer_id": self._indexer_id})
        if stored is None:
            self._initialize_configuration(initial)
            return False

        encoded_filter = stored.get("filter")
        if encoded_filter is None:
            raise RuntimeError("indexer filter is missing")
        initial.filter.parse(encoded_filter)

        cursor = stored.get("cursor")
        if cursor is not None:
            initial.starting_cursor = cursor_utils.from_json(cursor)

        return True

    def _update_filter(self, filter: Filter, session: Optional[ClientSession] = None):
        """Set the indexer filter, overriding the previous filter."""
        logger.debug("update stored filter")

        self.db["_apibara"].update_one(
            {"indexer_id": self._indexer_id},
            {"$set": {"filter": filter.encode()}},
            session=session,
        )

    def invalidate(self, cursor: Cursor, session: Optional[ClientSession] = None):
        """Invalidates all data generate after `cursor`."""
        logger.debug(f"invalidate data after {cursor}")
        if session is None:
            with self._mongo.start_session() as session:
                self._invalidate(cursor, session)
        else:
            self._invalidate(cursor, session)

    def _invalidate(self, cursor: Cursor, session: ClientSession):
        for collection in self.db.list_collections(session=session):
            name = collection["name"]
            if name.startswith("_"):
                continue
            # remove items inserted after block_number
            self.db[name].delete_many(
                {"_chain.valid_from": {"$gt": cursor.order_key}}, session=session
            )
            # rollback items updated after block_number
            self.db[name].update_many(
                {"_chain.valid_to": {"$gt": cursor.order_key}},
                {"$set": {"_chain.valid_to": None}},
                session=session,
            )

    def drop_database(self):
        logger.debug("dropping database %s", self.db_name)
        self._mongo.drop_database(self.db_name)

    def _update_cursor(self, cursor: Cursor, session: Optional[ClientSession] = None):
        self.db["_apibara"].update_one(
            {"indexer_id": self._indexer_id},
            {"$set": {"cursor": cursor_utils.to_json(cursor)}},
            upsert=True,
            session=session,
        )


class ReadOnlyStorage:
    """Chain-aware document storage, read methods."""

    def __init__(
        self, db: Database, *, session: Optional[ClientSession] = None
    ) -> None:
        self._db = db
        self._session = session

    async def find_one(
        self, collection: str, filter: DocumentFilter
    ) -> Optional[Document]:
        """Find the first document in `collection` matching `filter`."""
        self._add_current_block_to_filter(filter)
        return self._db[collection].find_one(filter, session=self._session)

    async def find(
        self,
        collection: str,
        filter: DocumentFilter,
        sort: Optional[dict[str, int]] = None,
        projection: Optional[Projection] = None,
        skip: int = 0,
        limit: int = 0,
    ) -> Iterable[dict]:
        """Find all documents in `collection` matching `filter`.

        Arguments
        ---------
        - `collection`: the collection,
        - `filter`: the filter,
        - `sort`: keys used for sorting, e.g. `{"a": -1}` sorts documents by key `a` in descending order,
        - `project`: filter document keys to reduce the document size,
        - `skip`: number of documents to skip,
        - `limit`: maximum number of documents returned.
        """
        self._add_current_block_to_filter(filter)
        cursor = self._db[collection].find(
            filter, projection, skip, limit, session=self._session
        )
        if sort is not None:
            for field, order in sort.items():
                cursor = cursor.sort(field, order)
        return cursor

    def _add_current_block_to_filter(self, filter: DocumentFilter):
        filter["_chain.valid_to"] = None


class Storage(ReadOnlyStorage):
    """Chain-aware document storage."""

    def __init__(
        self, db: Database, cursor: Cursor, *, session: Optional[ClientSession] = None
    ) -> None:
        super().__init__(db, session=session)
        self._cursor = cursor

    async def insert_one(self, collection: str, doc: Document):
        """Insert `doc` into `collection`."""
        self._add_chain_information(doc)
        self._db[collection].insert_one(doc, session=self._session)

    async def insert_many(self, collection: str, docs: Iterable[Document]):
        """Insert multiple `docs` into `collection`."""
        for doc in docs:
            self._add_chain_information(doc)
        self._db[collection].insert_many(docs, session=self._session)

    async def delete_one(self, collection: str, filter: DocumentFilter):
        """ "Delete the first document in `collection` matching `filter`."""
        self._add_current_block_to_filter(filter)
        self._db[collection].update_one(
            filter,
            {"$set": {"_chain.valid_to": self._cursor.order_key}},
            session=self._session,
        )

    async def delete_many(self, collection: str, filter: DocumentFilter):
        """Delete all documents in `collection` matching `filter`."""
        self._add_current_block_to_filter(filter)
        self._db[collection].update_many(
            filter,
            {"$set": {"_chain.valid_to": self._cursor.order_key}},
            session=self._session,
        )

    async def find_one_and_replace(
        self,
        collection: str,
        filter: DocumentFilter,
        replacement: Document,
        upsert: bool = False,
    ):
        """Replace the first document in `collection` matching `filter` with `replacement`.
        If `upsert = True`, insert `replacement` even if no document matched the `filter`.
        """
        # Step 1. Update the old document (if any) by clamping its validity range
        self._add_current_block_to_filter(filter)
        existing = self._db[collection].find_one_and_update(
            filter,
            {"$set": {"_chain.valid_to": self._cursor.order_key}},
            session=self._session,
        )

        # Step 2. Insert the new document.
        # Insert only if the existing document exists or if upsert.
        if existing is not None or upsert:
            await self.insert_one(collection, replacement)

        return existing

    async def find_one_and_update(
        self, collection: str, filter: DocumentFilter, update: Update
    ):
        """Update the first document in `collection` matching `filter` with `update`."""
        # Step 1. Update the old document (if any) by clamping its validity range
        self._add_current_block_to_filter(filter)
        existing = self._db[collection].find_one_and_update(
            filter,
            {"$set": {"_chain.valid_to": self._cursor.order_key}},
            session=self._session,
        )

        # Step 2. To simulate an update, first insert then call update on it.
        if existing is not None:
            del existing["_id"]
            del existing["_chain"]
            await self.insert_one(collection, existing)
            self._db[collection].update_one(filter, update, session=self._session)

        return existing

    def _add_chain_information(self, doc: Document):
        doc["_chain"] = {"valid_from": self._cursor.order_key, "valid_to": None}



from typing import ClassVar as _ClassVar
from typing import Iterable as _Iterable
from typing import Mapping as _Mapping
from typing import Optional as _Optional
from typing import Union as _Union

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import containers as _containers

import apibara.starknet.proto.types_pb2 as _types_pb2

DESCRIPTOR: _descriptor.FileDescriptor

class DeclareTransactionFilter(_message.Message):
    __slots__ = ["class_hash", "sender_address"]
    CLASS_HASH_FIELD_NUMBER: _ClassVar[int]
    SENDER_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    class_hash: _types_pb2.FieldElement
    sender_address: _types_pb2.FieldElement
    def __init__(
        self,
        class_hash: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        sender_address: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
    ) -> None: ...

class DeclaredContractFilter(_message.Message):
    __slots__ = ["class_hash"]
    CLASS_HASH_FIELD_NUMBER: _ClassVar[int]
    class_hash: _types_pb2.FieldElement
    def __init__(
        self, class_hash: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...
    ) -> None: ...

class DeployAccountTransactionFilter(_message.Message):
    __slots__ = ["class_hash", "constructor_calldata", "contract_address_salt"]
    CLASS_HASH_FIELD_NUMBER: _ClassVar[int]
    CONSTRUCTOR_CALLDATA_FIELD_NUMBER: _ClassVar[int]
    CONTRACT_ADDRESS_SALT_FIELD_NUMBER: _ClassVar[int]
    class_hash: _types_pb2.FieldElement
    constructor_calldata: _containers.RepeatedCompositeFieldContainer[
        _types_pb2.FieldElement
    ]
    contract_address_salt: _types_pb2.FieldElement
    def __init__(
        self,
        contract_address_salt: _Optional[
            _Union[_types_pb2.FieldElement, _Mapping]
        ] = ...,
        class_hash: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        constructor_calldata: _Optional[
            _Iterable[_Union[_types_pb2.FieldElement, _Mapping]]
        ] = ...,
    ) -> None: ...

class DeployTransactionFilter(_message.Message):
    __slots__ = ["class_hash", "constructor_calldata", "contract_address_salt"]
    CLASS_HASH_FIELD_NUMBER: _ClassVar[int]
    CONSTRUCTOR_CALLDATA_FIELD_NUMBER: _ClassVar[int]
    CONTRACT_ADDRESS_SALT_FIELD_NUMBER: _ClassVar[int]
    class_hash: _types_pb2.FieldElement
    constructor_calldata: _containers.RepeatedCompositeFieldContainer[
        _types_pb2.FieldElement
    ]
    contract_address_salt: _types_pb2.FieldElement
    def __init__(
        self,
        contract_address_salt: _Optional[
            _Union[_types_pb2.FieldElement, _Mapping]
        ] = ...,
        class_hash: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        constructor_calldata: _Optional[
            _Iterable[_Union[_types_pb2.FieldElement, _Mapping]]
        ] = ...,
    ) -> None: ...

class DeployedContractFilter(_message.Message):
    __slots__ = ["class_hash", "contract_address"]
    CLASS_HASH_FIELD_NUMBER: _ClassVar[int]
    CONTRACT_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    class_hash: _types_pb2.FieldElement
    contract_address: _types_pb2.FieldElement
    def __init__(
        self,
        contract_address: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        class_hash: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
    ) -> None: ...

class EventFilter(_message.Message):
    __slots__ = ["data", "from_address", "keys"]
    DATA_FIELD_NUMBER: _ClassVar[int]
    FROM_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    KEYS_FIELD_NUMBER: _ClassVar[int]
    data: _containers.RepeatedCompositeFieldContainer[_types_pb2.FieldElement]
    from_address: _types_pb2.FieldElement
    keys: _containers.RepeatedCompositeFieldContainer[_types_pb2.FieldElement]
    def __init__(
        self,
        from_address: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        keys: _Optional[_Iterable[_Union[_types_pb2.FieldElement, _Mapping]]] = ...,
        data: _Optional[_Iterable[_Union[_types_pb2.FieldElement, _Mapping]]] = ...,
    ) -> None: ...

class Filter(_message.Message):
    __slots__ = ["events", "header", "messages", "state_update", "transactions"]
    EVENTS_FIELD_NUMBER: _ClassVar[int]
    HEADER_FIELD_NUMBER: _ClassVar[int]
    MESSAGES_FIELD_NUMBER: _ClassVar[int]
    STATE_UPDATE_FIELD_NUMBER: _ClassVar[int]
    TRANSACTIONS_FIELD_NUMBER: _ClassVar[int]
    events: _containers.RepeatedCompositeFieldContainer[EventFilter]
    header: HeaderFilter
    messages: _containers.RepeatedCompositeFieldContainer[L2ToL1MessageFilter]
    state_update: StateUpdateFilter
    transactions: _containers.RepeatedCompositeFieldContainer[TransactionFilter]
    def __init__(
        self,
        header: _Optional[_Union[HeaderFilter, _Mapping]] = ...,
        transactions: _Optional[_Iterable[_Union[TransactionFilter, _Mapping]]] = ...,
        state_update: _Optional[_Union[StateUpdateFilter, _Mapping]] = ...,
        events: _Optional[_Iterable[_Union[EventFilter, _Mapping]]] = ...,
        messages: _Optional[_Iterable[_Union[L2ToL1MessageFilter, _Mapping]]] = ...,
    ) -> None: ...

class HeaderFilter(_message.Message):
    __slots__ = ["weak"]
    WEAK_FIELD_NUMBER: _ClassVar[int]
    weak: bool
    def __init__(self, weak: bool = ...) -> None: ...

class InvokeTransactionV0Filter(_message.Message):
    __slots__ = ["calldata", "contract_address", "entry_point_selector"]
    CALLDATA_FIELD_NUMBER: _ClassVar[int]
    CONTRACT_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    ENTRY_POINT_SELECTOR_FIELD_NUMBER: _ClassVar[int]
    calldata: _containers.RepeatedCompositeFieldContainer[_types_pb2.FieldElement]
    contract_address: _types_pb2.FieldElement
    entry_point_selector: _types_pb2.FieldElement
    def __init__(
        self,
        contract_address: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        entry_point_selector: _Optional[
            _Union[_types_pb2.FieldElement, _Mapping]
        ] = ...,
        calldata: _Optional[_Iterable[_Union[_types_pb2.FieldElement, _Mapping]]] = ...,
    ) -> None: ...

class InvokeTransactionV1Filter(_message.Message):
    __slots__ = ["calldata", "sender_address"]
    CALLDATA_FIELD_NUMBER: _ClassVar[int]
    SENDER_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    calldata: _containers.RepeatedCompositeFieldContainer[_types_pb2.FieldElement]
    sender_address: _types_pb2.FieldElement
    def __init__(
        self,
        sender_address: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        calldata: _Optional[_Iterable[_Union[_types_pb2.FieldElement, _Mapping]]] = ...,
    ) -> None: ...

class L1HandlerTransactionFilter(_message.Message):
    __slots__ = ["calldata", "contract_address", "entry_point_selector"]
    CALLDATA_FIELD_NUMBER: _ClassVar[int]
    CONTRACT_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    ENTRY_POINT_SELECTOR_FIELD_NUMBER: _ClassVar[int]
    calldata: _containers.RepeatedCompositeFieldContainer[_types_pb2.FieldElement]
    contract_address: _types_pb2.FieldElement
    entry_point_selector: _types_pb2.FieldElement
    def __init__(
        self,
        contract_address: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        entry_point_selector: _Optional[
            _Union[_types_pb2.FieldElement, _Mapping]
        ] = ...,
        calldata: _Optional[_Iterable[_Union[_types_pb2.FieldElement, _Mapping]]] = ...,
    ) -> None: ...

class L2ToL1MessageFilter(_message.Message):
    __slots__ = ["payload", "to_address"]
    PAYLOAD_FIELD_NUMBER: _ClassVar[int]
    TO_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    payload: _containers.RepeatedCompositeFieldContainer[_types_pb2.FieldElement]
    to_address: _types_pb2.FieldElement
    def __init__(
        self,
        to_address: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        payload: _Optional[_Iterable[_Union[_types_pb2.FieldElement, _Mapping]]] = ...,
    ) -> None: ...

class NonceUpdateFilter(_message.Message):
    __slots__ = ["contract_address", "nonce"]
    CONTRACT_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    NONCE_FIELD_NUMBER: _ClassVar[int]
    contract_address: _types_pb2.FieldElement
    nonce: _types_pb2.FieldElement
    def __init__(
        self,
        contract_address: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
        nonce: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
    ) -> None: ...

class StateUpdateFilter(_message.Message):
    __slots__ = ["declared_contracts", "deployed_contracts", "nonces", "storage_diffs"]
    DECLARED_CONTRACTS_FIELD_NUMBER: _ClassVar[int]
    DEPLOYED_CONTRACTS_FIELD_NUMBER: _ClassVar[int]
    NONCES_FIELD_NUMBER: _ClassVar[int]
    STORAGE_DIFFS_FIELD_NUMBER: _ClassVar[int]
    declared_contracts: _containers.RepeatedCompositeFieldContainer[
        DeclaredContractFilter
    ]
    deployed_contracts: _containers.RepeatedCompositeFieldContainer[
        DeployedContractFilter
    ]
    nonces: _containers.RepeatedCompositeFieldContainer[NonceUpdateFilter]
    storage_diffs: _containers.RepeatedCompositeFieldContainer[StorageDiffFilter]
    def __init__(
        self,
        storage_diffs: _Optional[_Iterable[_Union[StorageDiffFilter, _Mapping]]] = ...,
        declared_contracts: _Optional[
            _Iterable[_Union[DeclaredContractFilter, _Mapping]]
        ] = ...,
        deployed_contracts: _Optional[
            _Iterable[_Union[DeployedContractFilter, _Mapping]]
        ] = ...,
        nonces: _Optional[_Iterable[_Union[NonceUpdateFilter, _Mapping]]] = ...,
    ) -> None: ...

class StorageDiffFilter(_message.Message):
    __slots__ = ["contract_address"]
    CONTRACT_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    contract_address: _types_pb2.FieldElement
    def __init__(
        self,
        contract_address: _Optional[_Union[_types_pb2.FieldElement, _Mapping]] = ...,
    ) -> None: ...

class TransactionFilter(_message.Message):
    __slots__ = [
        "declare",
        "deploy",
        "deploy_account",
        "invoke_v0",
        "invoke_v1",
        "l1_handler",
    ]
    DECLARE_FIELD_NUMBER: _ClassVar[int]
    DEPLOY_ACCOUNT_FIELD_NUMBER: _ClassVar[int]
    DEPLOY_FIELD_NUMBER: _ClassVar[int]
    INVOKE_V0_FIELD_NUMBER: _ClassVar[int]
    INVOKE_V1_FIELD_NUMBER: _ClassVar[int]
    L1_HANDLER_FIELD_NUMBER: _ClassVar[int]
    declare: DeclareTransactionFilter
    deploy: DeployTransactionFilter
    deploy_account: DeployAccountTransactionFilter
    invoke_v0: InvokeTransactionV0Filter
    invoke_v1: InvokeTransactionV1Filter
    l1_handler: L1HandlerTransactionFilter
    def __init__(
        self,
        invoke_v0: _Optional[_Union[InvokeTransactionV0Filter, _Mapping]] = ...,
        invoke_v1: _Optional[_Union[InvokeTransactionV1Filter, _Mapping]] = ...,
        deploy: _Optional[_Union[DeployTransactionFilter, _Mapping]] = ...,
        declare: _Optional[_Union[DeclareTransactionFilter, _Mapping]] = ...,
        l1_handler: _Optional[_Union[L1HandlerTransactionFilter, _Mapping]] = ...,
        deploy_account: _Optional[
            _Union[DeployAccountTransactionFilter, _Mapping]
        ] = ...,
    ) -> None: ...



from dataclasses import dataclass
from typing import ClassVar

from .client import (
    BearerTokenAuth,
    StreamClient,
    StreamIter,
    StreamService,
    credentials_with_auth_token,
)
from .proto.stream_pb2 import (
    Cursor,
    DataFinality,
    StreamDataRequest,
    StreamDataResponse,
)


@dataclass
class StarkNetStreamAddress:
    Mainnet: ClassVar[str] = "mainnet.starknet.a5a.ch"
    Goerli: ClassVar[str] = "goerli.starknet.a5a.ch"
    Goerli2: ClassVar[str] = "goerli-2.starknet.a5a.ch"


@dataclass
class StreamAddress:
    """Defines well-known addresses for the Apibara Stream service."""

    StarkNet: ClassVar[StarkNetStreamAddress] = StarkNetStreamAddress



from apibara.starknet.proto.types_pb2 import FieldElement


def to_int(felt: FieldElement) -> int:
    """
    Convert the wire-encoded field element to an int.

    Arguments
    ---------
    felt : FieldElement
        the protobuf field element object.
    """
    return (felt.lo_lo << 192) + (felt.lo_hi << 128) + (felt.hi_lo << 64) + felt.hi_hi


def to_hex(felt: FieldElement) -> str:
    """
    Convert the wire-encoded field element to hex.

    Arguments
    ---------
    felt : FieldElement
        the protobuf field element object.
    """
    return "0x" + _normalize_hex(hex(to_int(felt)))


def from_int(num: int) -> FieldElement:
    """
    Create a new wire-encoded field element from an int.

    Arguments
    ---------
    num : int
        the number
    """
    return from_hex(hex(num))


def from_hex(h: str) -> FieldElement:
    """
    Create a new wire-encoded field element from a hex string.

    Arguments
    ---------
    h : str
        the hex string. can start with 0.
    """
    h = _normalize_hex(h)
    s = len(h)
    lo_lo = int(h[0:16], 16)
    lo_hi = int(h[16:32], 16)
    hi_lo = int(h[32:48], 16)
    hi_hi = int(h[48:64], 16)
    return FieldElement(
        lo_lo=lo_lo,
        lo_hi=lo_hi,
        hi_lo=hi_lo,
        hi_hi=hi_hi,
    )


def _normalize_hex(h: str) -> str:
    return h.replace("0x", "").rjust(64, "0")



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Apibara Stream service."""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _DataFinality:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _DataFinalityEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[
        _DataFinality.ValueType
    ],
    builtins.type,
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    DATA_STATUS_UNKNOWN: _DataFinality.ValueType  # 0
    DATA_STATUS_PENDING: _DataFinality.ValueType  # 1
    """Data was received, but is not part of the canonical chain yet."""
    DATA_STATUS_ACCEPTED: _DataFinality.ValueType  # 2
    """Data is now part of the canonical chain, but could still be invalidated."""
    DATA_STATUS_FINALIZED: _DataFinality.ValueType  # 3
    """Data is finalized and cannot be invalidated."""

class DataFinality(_DataFinality, metaclass=_DataFinalityEnumTypeWrapper):
    """Data finality."""

DATA_STATUS_UNKNOWN: DataFinality.ValueType  # 0
DATA_STATUS_PENDING: DataFinality.ValueType  # 1
"""Data was received, but is not part of the canonical chain yet."""
DATA_STATUS_ACCEPTED: DataFinality.ValueType  # 2
"""Data is now part of the canonical chain, but could still be invalidated."""
DATA_STATUS_FINALIZED: DataFinality.ValueType  # 3
"""Data is finalized and cannot be invalidated."""
global___DataFinality = DataFinality

@typing_extensions.final
class StreamDataRequest(google.protobuf.message.Message):
    """Request data to be streamed."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STREAM_ID_FIELD_NUMBER: builtins.int
    BATCH_SIZE_FIELD_NUMBER: builtins.int
    STARTING_CURSOR_FIELD_NUMBER: builtins.int
    FINALITY_FIELD_NUMBER: builtins.int
    FILTER_FIELD_NUMBER: builtins.int
    stream_id: builtins.int
    """Used by the client to uniquely identify a stream.
    All streams use `stream_id = 0` by default.
    """
    batch_size: builtins.int
    """How many items to send in a single response."""
    @property
    def starting_cursor(self) -> global___Cursor:
        """Start streaming from the provided cursor."""
    finality: global___DataFinality.ValueType
    """Return data with the specified finality.
    If not specified, defaults to `DATA_STATUS_ACCEPTED`.
    """
    filter: builtins.bytes
    """Return data according to the stream-specific filter."""
    def __init__(
        self,
        *,
        stream_id: builtins.int | None = ...,
        batch_size: builtins.int | None = ...,
        starting_cursor: global___Cursor | None = ...,
        finality: global___DataFinality.ValueType | None = ...,
        filter: builtins.bytes = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing_extensions.Literal[
            "_batch_size",
            b"_batch_size",
            "_finality",
            b"_finality",
            "_stream_id",
            b"_stream_id",
            "batch_size",
            b"batch_size",
            "finality",
            b"finality",
            "starting_cursor",
            b"starting_cursor",
            "stream_id",
            b"stream_id",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "_batch_size",
            b"_batch_size",
            "_finality",
            b"_finality",
            "_stream_id",
            b"_stream_id",
            "batch_size",
            b"batch_size",
            "filter",
            b"filter",
            "finality",
            b"finality",
            "starting_cursor",
            b"starting_cursor",
            "stream_id",
            b"stream_id",
        ],
    ) -> None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing_extensions.Literal["_batch_size", b"_batch_size"]
    ) -> typing_extensions.Literal["batch_size"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing_extensions.Literal["_finality", b"_finality"]
    ) -> typing_extensions.Literal["finality"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing_extensions.Literal["_stream_id", b"_stream_id"]
    ) -> typing_extensions.Literal["stream_id"] | None: ...

global___StreamDataRequest = StreamDataRequest

@typing_extensions.final
class StreamDataResponse(google.protobuf.message.Message):
    """Contains the data requested from the client."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STREAM_ID_FIELD_NUMBER: builtins.int
    INVALIDATE_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    HEARTBEAT_FIELD_NUMBER: builtins.int
    stream_id: builtins.int
    """The stream id."""
    @property
    def invalidate(self) -> global___Invalidate: ...
    @property
    def data(self) -> global___Data: ...
    @property
    def heartbeat(self) -> global___Heartbeat: ...
    def __init__(
        self,
        *,
        stream_id: builtins.int = ...,
        invalidate: global___Invalidate | None = ...,
        data: global___Data | None = ...,
        heartbeat: global___Heartbeat | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing_extensions.Literal[
            "data",
            b"data",
            "heartbeat",
            b"heartbeat",
            "invalidate",
            b"invalidate",
            "message",
            b"message",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "data",
            b"data",
            "heartbeat",
            b"heartbeat",
            "invalidate",
            b"invalidate",
            "message",
            b"message",
            "stream_id",
            b"stream_id",
        ],
    ) -> None: ...
    def WhichOneof(
        self, oneof_group: typing_extensions.Literal["message", b"message"]
    ) -> typing_extensions.Literal["invalidate", "data", "heartbeat"] | None: ...

global___StreamDataResponse = StreamDataResponse

@typing_extensions.final
class Cursor(google.protobuf.message.Message):
    """A cursor over the stream content."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ORDER_KEY_FIELD_NUMBER: builtins.int
    UNIQUE_KEY_FIELD_NUMBER: builtins.int
    order_key: builtins.int
    """Key used for ordering messages in the stream."""
    unique_key: builtins.bytes
    """Key used to discriminate branches in the stream."""
    def __init__(
        self,
        *,
        order_key: builtins.int = ...,
        unique_key: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "order_key", b"order_key", "unique_key", b"unique_key"
        ],
    ) -> None: ...

global___Cursor = Cursor

@typing_extensions.final
class Invalidate(google.protobuf.message.Message):
    """Invalidate data after the given cursor."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CURSOR_FIELD_NUMBER: builtins.int
    @property
    def cursor(self) -> global___Cursor:
        """The cursor of the message before the now invalid data."""
    def __init__(
        self,
        *,
        cursor: global___Cursor | None = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing_extensions.Literal["cursor", b"cursor"]
    ) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["cursor", b"cursor"]
    ) -> None: ...

global___Invalidate = Invalidate

@typing_extensions.final
class Data(google.protobuf.message.Message):
    """A batch of data."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    END_CURSOR_FIELD_NUMBER: builtins.int
    FINALITY_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    CURSOR_FIELD_NUMBER: builtins.int
    @property
    def end_cursor(self) -> global___Cursor:
        """Cursor of the last item in the batch."""
    finality: global___DataFinality.ValueType
    """The finality status of the data in the batch."""
    @property
    def data(
        self,
    ) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.bytes
    ]:
        """The stream data."""
    @property
    def cursor(self) -> global___Cursor:
        """Cursor used to produced the batch."""
    def __init__(
        self,
        *,
        end_cursor: global___Cursor | None = ...,
        finality: global___DataFinality.ValueType = ...,
        data: collections.abc.Iterable[builtins.bytes] | None = ...,
        cursor: global___Cursor | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing_extensions.Literal[
            "cursor", b"cursor", "end_cursor", b"end_cursor"
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "cursor",
            b"cursor",
            "data",
            b"data",
            "end_cursor",
            b"end_cursor",
            "finality",
            b"finality",
        ],
    ) -> None: ...

global___Data = Data

@typing_extensions.final
class Heartbeat(google.protobuf.message.Message):
    """Sent to clients to check if stream is still connected."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___Heartbeat = Heartbeat



from apibara.protocol.proto.stream_pb2 import Cursor


def to_json(cursor: Cursor) -> dict:
    """Converts cursor to a json object."""
    return {"order_key": cursor.order_key, "unique_key": cursor.unique_key.hex()}


def from_json(cursor: dict) -> Cursor:
    """Returns a `Cursor` from its json representation."""
    if "unique_key" in cursor:
        unique_key = bytes.fromhex(cursor["unique_key"])
    else:
        unique_key = b""

    return Cursor(order_key=cursor["order_key"], unique_key=unique_key)



from typing import ClassVar as _ClassVar
from typing import Optional as _Optional

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message

DESCRIPTOR: _descriptor.FileDescriptor

class FieldElement(_message.Message):
    __slots__ = ["hi_hi", "hi_lo", "lo_hi", "lo_lo"]
    HI_HI_FIELD_NUMBER: _ClassVar[int]
    HI_LO_FIELD_NUMBER: _ClassVar[int]
    LO_HI_FIELD_NUMBER: _ClassVar[int]
    LO_LO_FIELD_NUMBER: _ClassVar[int]
    hi_hi: int
    hi_lo: int
    lo_hi: int
    lo_lo: int
    def __init__(
        self,
        lo_lo: _Optional[int] = ...,
        lo_hi: _Optional[int] = ...,
        hi_lo: _Optional[int] = ...,
        hi_hi: _Optional[int] = ...,
    ) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: stream.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x0cstream.proto\x12\x15\x61pibara.node.v1alpha2"\xf2\x01\n\x11StreamDataRequest\x12\x16\n\tstream_id\x18\x01 \x01(\x04H\x00\x88\x01\x01\x12\x17\n\nbatch_size\x18\x02 \x01(\x04H\x01\x88\x01\x01\x12\x36\n\x0fstarting_cursor\x18\x03 \x01(\x0b\x32\x1d.apibara.node.v1alpha2.Cursor\x12:\n\x08\x66inality\x18\x04 \x01(\x0e\x32#.apibara.node.v1alpha2.DataFinalityH\x02\x88\x01\x01\x12\x0e\n\x06\x66ilter\x18\x05 \x01(\x0c\x42\x0c\n\n_stream_idB\r\n\x0b_batch_sizeB\x0b\n\t_finality"\xcf\x01\n\x12StreamDataResponse\x12\x11\n\tstream_id\x18\x01 \x01(\x04\x12\x37\n\ninvalidate\x18\x02 \x01(\x0b\x32!.apibara.node.v1alpha2.InvalidateH\x00\x12+\n\x04\x64\x61ta\x18\x03 \x01(\x0b\x32\x1b.apibara.node.v1alpha2.DataH\x00\x12\x35\n\theartbeat\x18\x04 \x01(\x0b\x32 .apibara.node.v1alpha2.HeartbeatH\x00\x42\t\n\x07message"/\n\x06\x43ursor\x12\x11\n\torder_key\x18\x01 \x01(\x04\x12\x12\n\nunique_key\x18\x02 \x01(\x0c";\n\nInvalidate\x12-\n\x06\x63ursor\x18\x01 \x01(\x0b\x32\x1d.apibara.node.v1alpha2.Cursor"\xad\x01\n\x04\x44\x61ta\x12\x31\n\nend_cursor\x18\x01 \x01(\x0b\x32\x1d.apibara.node.v1alpha2.Cursor\x12\x35\n\x08\x66inality\x18\x02 \x01(\x0e\x32#.apibara.node.v1alpha2.DataFinality\x12\x0c\n\x04\x64\x61ta\x18\x03 \x03(\x0c\x12-\n\x06\x63ursor\x18\x04 \x01(\x0b\x32\x1d.apibara.node.v1alpha2.Cursor"\x0b\n\tHeartbeat*u\n\x0c\x44\x61taFinality\x12\x17\n\x13\x44\x41TA_STATUS_UNKNOWN\x10\x00\x12\x17\n\x13\x44\x41TA_STATUS_PENDING\x10\x01\x12\x18\n\x14\x44\x41TA_STATUS_ACCEPTED\x10\x02\x12\x19\n\x15\x44\x41TA_STATUS_FINALIZED\x10\x03\x32o\n\x06Stream\x12\x65\n\nStreamData\x12(.apibara.node.v1alpha2.StreamDataRequest\x1a).apibara.node.v1alpha2.StreamDataResponse(\x01\x30\x01\x62\x06proto3'
)

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "stream_pb2", globals())
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _DATAFINALITY._serialized_start = 793
    _DATAFINALITY._serialized_end = 910
    _STREAMDATAREQUEST._serialized_start = 40
    _STREAMDATAREQUEST._serialized_end = 282
    _STREAMDATARESPONSE._serialized_start = 285
    _STREAMDATARESPONSE._serialized_end = 492
    _CURSOR._serialized_start = 494
    _CURSOR._serialized_end = 541
    _INVALIDATE._serialized_start = 543
    _INVALIDATE._serialized_end = 602
    _DATA._serialized_start = 605
    _DATA._serialized_end = 778
    _HEARTBEAT._serialized_start = 780
    _HEARTBEAT._serialized_end = 791
    _STREAM._serialized_start = 912
    _STREAM._serialized_end = 1023
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Apibara Stream service."""
import abc
import collections.abc
import grpc
import apibara.protocol.proto.stream_pb2 as stream_pb2

class StreamStub:
    def __init__(self, channel: grpc.Channel) -> None: ...
    StreamData: grpc.StreamStreamMultiCallable[
        stream_pb2.StreamDataRequest,
        stream_pb2.StreamDataResponse,
    ]
    """Stream data from the node."""

class StreamServicer(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def StreamData(
        self,
        request_iterator: collections.abc.Iterator[stream_pb2.StreamDataRequest],
        context: grpc.ServicerContext,
    ) -> collections.abc.Iterator[stream_pb2.StreamDataResponse]:
        """Stream data from the node."""

def add_StreamServicer_to_server(
    servicer: StreamServicer, server: grpc.Server
) -> None: ...



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['apibara',
 'apibara.indexer',
 'apibara.protocol',
 'apibara.protocol.proto',
 'apibara.starknet',
 'apibara.starknet.proto']

package_data = \
{'': ['*']}

install_requires = \
['grpcio>=1.50,<2.0', 'protobuf>=4.20,<5']

extras_require = \
{':extra == "indexer"': ['pymongo>=4.3.3,<5.0.0']}

setup_kwargs = {
    'name': 'apibara',
    'version': '0.6.6',
    'description': 'Apibara cliend SDK. Stream and transform on-chain data with Python.',
    'long_description': 'Apibara Python SDK\n==================\n\n.. warning::\n    This SDK is alpha software. The API will change drastically until the beta.\n\n    `Open an issue on GitHub <https://github.com/apibara/python-sdk>`_ to report bugs or provide feedback.\n\n\nBuild web3-powered applications in Python. \n\nDevelopment\n-----------\n\nInstall all dependencies with:\n\n.. code::\n\n    poetry install\n\nRun tests with:\n\n.. code::\n\n    poetry run pytest tests\n\nFormat code with:\n\n.. code::\n\n    poetry run black src examples test\n    poetry run isort src examples test\n\nTo update the protobuf definitions:\n\n.. code::\n\n    protoc -I=protos/starknet/ \\\n        --python_out=src/apibara/starknet/proto/ \\\n        --pyi_out=src/apibara/starknet/proto protos/starknet/*\n\n\nLicense\n-------\n\n   Copyright 2022 GNC Labs Limited\n\n   Licensed under the Apache License, Version 2.0 (the "License");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an "AS IS" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.\n',
    'author': 'Francesco Ceccon',
    'author_email': 'francesco@apibara.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://www.apibara.com',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: types.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x0btypes.proto\x12\x19\x61pibara.starknet.v1alpha2"J\n\x0c\x46ieldElement\x12\r\n\x05lo_lo\x18\x01 \x01(\x06\x12\r\n\x05lo_hi\x18\x02 \x01(\x06\x12\r\n\x05hi_lo\x18\x03 \x01(\x06\x12\r\n\x05hi_hi\x18\x04 \x01(\x06\x62\x06proto3'
)

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "types_pb2", globals())
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _FIELDELEMENT._serialized_start = 42
    _FIELDELEMENT._serialized_end = 116
# @@protoc_insertion_point(module_scope)
